"use strict";
/********************************************************************************
 * Copyright (C) 2021 Jon Radchenko and others.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * This Source Code may also be made available under the following Secondary
 * Licenses when the conditions for such availability set forth in the Eclipse
 * Public License v. 2.0 are satisfied: GNU General Public License, version 2
 * with the GNU Classpath Exception which is available at
 * https://www.gnu.org/software/classpath/license.html.
 *
 * SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
 ********************************************************************************/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RStudioOpenHandler = void 0;
var core_1 = require("@theia/core");
var inversify_1 = require("inversify");
var browser_1 = require("@theia/core/lib/browser");
var server_proxy_instance_manager_1 = require("theia-server-proxy-extension/lib/browser/server-proxy-instance-manager");
var workspace_service_1 = require("@theia/workspace/lib/browser/workspace-service");
var const_1 = require("../common/const");
var server_proxy_open_handler_1 = require("theia-server-proxy-extension/lib/browser/server-proxy-open-handler");
var RStudioOpenHandler = /** @class */ (function () {
    function RStudioOpenHandler() {
        this.id = 'rstudio.openhandler';
        this.label = 'Open in RStudio';
    }
    RStudioOpenHandler.prototype.canHandle = function (uri) {
        try {
            var lowercaseExtension = uri.path.ext.toLocaleLowerCase();
            if (lowercaseExtension == ".rmd" || lowercaseExtension == ".r") {
                return 500;
            }
            return 0;
        }
        catch (_a) {
            return 0;
        }
    };
    RStudioOpenHandler.prototype.fallback = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var openHandler;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.openerService.getOpeners(uri)];
                    case 1:
                        openHandler = (_a.sent())
                            .find(function (p) { return p != _this; });
                        return [2 /*return*/, openHandler === null || openHandler === void 0 ? void 0 : openHandler.open(uri)];
                }
            });
        });
    };
    RStudioOpenHandler.prototype.open = function (uri) {
        return __awaiter(this, void 0, void 0, function () {
            var workspace, relativeFilePath, instance, widget, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        workspace = this.workspaceService.getWorkspaceRootUri(uri);
                        if (!workspace) {
                            return [2 /*return*/, this.fallback(uri)];
                        }
                        relativeFilePath = workspace.relative(uri);
                        if (!relativeFilePath) {
                            return [2 /*return*/, this.fallback(uri)];
                        }
                        return [4 /*yield*/, this.serverProxyInstanceManager.getOrCreateInstance(const_1.Extension.ServerProxy, {})];
                    case 1:
                        instance = _a.sent();
                        if (!instance) {
                            this.logger.warn("RStudio instance did not start properly");
                            return [2 /*return*/, this.fallback(uri)];
                        }
                        return [4 /*yield*/, server_proxy_open_handler_1.ServerProxyOpenHandler.open(this.openerService, instance)];
                    case 2:
                        widget = _a.sent();
                        if (!widget) {
                            this.logger.warn("RStudio widget did not open");
                            return [2 /*return*/, this.fallback(uri)];
                        }
                        data = JSON.stringify({
                            method: "console_input",
                            //TODO: is this the proper escaping
                            params: ["rstudioapi::navigateToFile(\"" + uri.path.toString().replace('"', '\\"') + "\")", "", 0],
                        });
                        return [4 /*yield*/, fetch("/server-proxy/" + const_1.Extension.ID + "/" + instance.id + "/rpc/theia_remote", {
                                method: 'POST',
                                headers: {
                                    'Content-Type': 'application/json',
                                    'Content-Length': data.length.toString(),
                                },
                                body: data
                            })];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    __decorate([
        inversify_1.inject(workspace_service_1.WorkspaceService),
        __metadata("design:type", workspace_service_1.WorkspaceService)
    ], RStudioOpenHandler.prototype, "workspaceService", void 0);
    __decorate([
        inversify_1.inject(server_proxy_instance_manager_1.ServerProxyInstanceManager),
        __metadata("design:type", server_proxy_instance_manager_1.ServerProxyInstanceManager)
    ], RStudioOpenHandler.prototype, "serverProxyInstanceManager", void 0);
    __decorate([
        inversify_1.inject(browser_1.OpenerService),
        __metadata("design:type", Object)
    ], RStudioOpenHandler.prototype, "openerService", void 0);
    __decorate([
        inversify_1.inject(core_1.ILogger),
        __metadata("design:type", Object)
    ], RStudioOpenHandler.prototype, "logger", void 0);
    RStudioOpenHandler = __decorate([
        inversify_1.injectable()
    ], RStudioOpenHandler);
    return RStudioOpenHandler;
}());
exports.RStudioOpenHandler = RStudioOpenHandler;
//# sourceMappingURL=rstudio-open-handler.js.map